// Copyright (c) 2016, Thomas L. Falch
// For conditions of distribution and use, see the accompanying LICENSE and README files

// This file is part of the ImageCL source-to-source compiler
// developed at the Norwegian University of Science and technology


#ifndef KERNELINFO_H
#define KERNELINFO_H

#include "rose.h"

#include "footprintfinder.h"
#include "type.h"
#include "pragma.h"
#include "settings.h"

#include <string>
#include <map>

using namespace std;

class KernelInfo {
public:
    KernelInfo(SgProject *project, Settings settings);

    void scanForInfo();


    string getKernelName();

    set<string> *getGridArrays();

    set<string> *getReadOnlyArrays();

    set<string> *getConstantArrays();

    set<string> *getImageArrays();

    inline set<string> &getBlockArrays() {
        return blockArrays;
    }

    string getAGridArray();

    bool isGridArray(string arrayName);

    bool isImageArray(string arrayName);

    inline bool isBlockArray(const string &arrayName) {
        return this->blockArrays.count(arrayName) == 1;
    }

    inline bool isAliasArray(const string &array) {
        return this->aliasArrays.count(array) == 1;
    }

    bool isReadOnlyArray(string arrayName);

    bool isWriteOnlyArray(string arrayName);

    map<string, Footprint> getFootprintTable();

    BaseType getPixelType(string arr);

    set<Pragma> *getPragmas();

    vector<pair<int, vector<int> *>> *getForLoops();

    BoundaryCondition getBoundaryConditionForArray(string array);

    HaloSize getHaloSize(string array);

    bool needsMpiScatter(string argumentName);

    bool needsMpiBroadcast(string argumentName);

    bool needsGridPosArg();

    void printKernelInfo();

    inline bool isConstGridSize() {
        return this->constGridSize;
    }

    int gridSizeX = 0;
    int gridSizeY = 0;
    int gridSizeZ = 0;

    // Used for image or block types defined in the function body instead of the parameters.
    map<string, std::pair<string, SgExpression*>> aliasArrays;

private:
    void findKernelName();

    void findReadOnlyArrays();

    void findWriteOnlyArrays();

    void findConstantArrays();

    void findUnrollableLoops();

    void findImageAndBlockArrays();

    void setGridFromPragmas();

    void setBoundaryConditions();

    void parsePragmas();

    set<string> *findArraysReadFromWrittenTo(bool findArraysReadFrom, bool findArraysWrittenTo);

    SgProject *project;
    Settings settings;
    string kernelName;
    set<string> *gridArrays;
    map<string, Footprint> footprintTable;
    set<string> *allArrays;
    set<string> *readOnlyArrays;
    set<string> *writeOnlyArrays;
    set<string> *constantArrays;
    set<string> *imageArrays;
    set<string> blockArrays = set<string>();
    set<Pragma> *pragmas;
    vector<pair<int, vector<int> *>> *forLoops;
    map<string, BoundaryCondition> *boundaryConditions;
    map<string, BaseType> *pixelTypes;
    bool constGridSize = false;
};

#endif // KERNELINFO_H
