// Copyright (c) 2016, Thomas L. Falch
// For conditions of distribution and use, see the accompanying LICENSE and README files

// This file is part of the ImageCL source-to-source compiler
// developed at the Norwegian University of Science and technology


#include "settings.h"

#include <iostream>
#include <fstream>

using namespace std;

void Settings::readSettingsFromFile(string fileName) {
    ifstream file;
    file.open(fileName);
    if (!file.is_open()) {
        cout << "WARNING: No settings file found, using default values" << endl;
        return;
    }

    string line;
    while (getline(file, line)) {

        if (line[0] == '#') {
            continue;
        }

        auto split = line.find(':');
        string property = line.substr(0, split);
        string value = line.substr(split + 1, line.size());

        if (property == "GENERATE_FAST")
            generateFAST = stoi(value) != 0;

        if (property == "GENERATE_MPI")
            generateMPI = stoi(value) != 0;

        if (property == "GENERATE_OMP")
            generateOMP = stoi(value) != 0;

        if (property == "GENERATE_STANDALONE")
            generateStandalone = stoi(value) != 0;

        if (property == "GENERATE_TIMING")
            generateTiming = stoi(value) != 0;

        if (property == "DEVICE_ID")
            deviceId = stoi(value);

        if (property == "PLATFORM_ID")
            platformId = stoi(value);


        if (property == "N_LAUNCHES") {
            nLaunchesForTiming = stoi(value);
            if (nLaunchesForTiming <= 0) {
                nLaunchesForTiming = 1;
            }
        }

        if (property == "BOUNDARY_CONDITION") {
            if (value == "constant") {
                boundaryCondition = BoundaryCondition::CONSTANT;
            }
            if (value == "clamped") {
                boundaryCondition = BoundaryCondition::CLAMPED;
            }
            if (value == "none") {
                boundaryCondition = BoundaryCondition::NONE;
            }
        }

        if (property == "DEFAULT_PIXEL_TYPE") {
            defaultPixelType = Type::parseType(value);
        }
    }

    file.close();
}

void Settings::printSettings() {
    char esc_char = 27;
    cout << esc_char << "[1m" << "== Settings ==" << esc_char << "[0m" << endl;

    cout << "GENERATE_FAST: " << generateFAST << endl;
    cout << "GENERATE_STANDALONE: " << generateStandalone << endl;
    cout << "GENERATE_MPI: " << generateMPI << endl;
    cout << "GENERATE_OMP: " << generateOMP << endl;
    cout << "GENERATE_TIMING: " << generateTiming << endl;
    cout << "N_LAUNCHES: " << nLaunchesForTiming << endl;
    cout << "PLATFORM_ID: " << platformId << endl;
    cout << "DEVICE_ID: " << deviceId << endl;
    cout << "BOUNDARY_CONDITION: ";
    switch (boundaryCondition) {
        case BoundaryCondition::CLAMPED:
            cout << "CLAMPED";
            break;
        case BoundaryCondition::CONSTANT:
            cout << "CONSTANT";
            break;
        case BoundaryCondition::NONE:
            cout << "NONE";
            break;
    }
    cout << endl;
    cout << "DEFAULT_PIXEL_TYPE: " << Type::baseTypeToString(defaultPixelType) << endl;
    cout << "INPUT BASE NAME: " << inputBaseName << endl;
    cout << "GENERATE C: " << (this->language == Language::C) << endl;
    cout << "GENERATE CL: " << (this->language == Language::OpenCL) << endl;
    cout << "GENERATE CUDA: " << (this->language == Language::Cuda) << endl;
    cout << endl;
}
