// Copyright (c) 2016, Thomas L. Falch
// For conditions of distribution and use, see the accompanying LICENSE and README files

// This file is part of the ImageCL source-to-source compiler
// developed at the Norwegian University of Science and technology


#ifndef SETTINGS_H
#define SETTINGS_H

#include "type.h"

#include <string>

using namespace std;

enum class Language {
    C,
    OpenCL,
    Cuda,
};

enum class BoundaryCondition {
    CONSTANT, CLAMPED, NONE
};

class Settings {
public:
    Settings() = default;

    void readSettingsFromFile(string fileName);

    void printSettings();

    bool generateFAST = false;
    bool generateStandalone = true;
    bool generateMPI = false;
    bool generateOMP = false;

    BoundaryCondition boundaryCondition = BoundaryCondition::NONE;
    BaseType defaultPixelType = FLOAT;

    bool generateTiming = false;
    int nLaunchesForTiming = 1;
    int deviceId = 0;
    int platformId = 0;

    string inputBaseName;

    Language language = Language::OpenCL;
};

#endif // SETTINGS_H
